package net.jumperz.app.MGuardian;

import net.jumperz.util.*;
import net.jumperz.net.*;
import net.jumperz.net.exception.*;
import java.io.*;
import java.net.*;
import java.util.*;
import net.jumperz.app.MGuardian.rule.*;
import net.jumperz.app.MGuardian.plugin.*;

public final class MSession
implements MCommand, MObserver1, MSubject2
{
public  static final String SEC_LOG_PREFIX	= "Alert:";
private static final int LOG_BUFSIZE		= 300;
private static final int LOGFILENAME_BUFSIZE	= 40;
private static final int ALERT_LOG_LINE_BUFSIZE	= 200;
private static final String DECODE_CHARSET	= "ISO-8859-1";
private static final String HTTP		= "HTTP";
private static final String HTTPS		= "HTTPS";

private static String targetHost;
private static int targetPort;
private static boolean replaceHostField;
private static boolean replaceLocationField;
private static MHttpFilter requestFilter;
private static MHttpFilter responseFilter;
private static String targetPeer;
private static Map additionalResponseHeaderMap;
private static Map additionalRequestHeaderMap;
private static List requestRuleList;
private static List responseRuleList;
private static MTimer timer;
private static int requestHeaderTimeOut;
private static int requestBodyTimeOut;
private static int connectTimeOut;
private static List listenerList = new ArrayList();

private Socket clientSideSocket;
private Socket serverSideSocket;
private BufferedInputStream clientInputStream;
private OutputStream clientOutputStream;
private BufferedInputStream serverInputStream;
private OutputStream serverOutputStream;
private boolean keepAlive = true;
private String clientRemoteAddr;
private int clientRemotePort;
private String clientLocalAddr;
private int clientLocalPort;
private boolean timedOut = false;
private boolean isHttpsFlag = false;
private String protocol;
private MSubject2 subject2 = new MSubject2Impl();

	//loop
private boolean pass = false;
private int time;
private MResultCache resultCache;
private MHttpRequest request;
private MHttpResponse response;
private String sessionResult;
private Set passRuleSet;
private Exception exception;
private MAbstractRule triggeredRule;
private String logId;
private String pluginClassName;
private String pluginMessage;

//-----------------------------------------------------------------------------------
public MSession( Socket in_socket )
throws Exception
{
clientSideSocket = in_socket;
clientRemoteAddr = clientSideSocket.getInetAddress().getHostAddress();
clientRemotePort = clientSideSocket.getPort();
clientLocalAddr  = clientSideSocket.getLocalAddress().getHostAddress();
clientLocalPort  = clientSideSocket.getLocalPort();
getClientSideStreams();

isHttpsFlag		= ( clientSideSocket instanceof javax.net.ssl.SSLSocket );
if( isHttpsFlag == true )
	{
	protocol = HTTPS;
	}
else
	{
	protocol = HTTP;
	}
}
//-----------------------------------------------------------------------------------
private void setup()
{
timer.register1( this );
MAddressManager.getInstance().add( clientRemoteAddr );
}
//-----------------------------------------------------------------------------------
private void cleanup()
{
timer.removeObserver1( this );
MAddressManager.getInstance().remove( clientRemoteAddr );
}
//--------------------------------------------------------------------------------
public void execute()
{
try
	{
	setup();
	while( keepAlive )
		{
		resetLoopParameters();
		recvRequest();
		if( checkRequest() )
			{
			if( response == null )
				{
				filterRequest();
				modifyRequest();
				connect();
				sendRequest();
				recvResponse();
				}
			else
				{
				if( !response.isKeepAliveResponse() )
					{
					keepAlive = false;
					}
				}

			if( checkResponse() )
				{
				filterResponse();
				modifyResponse();
				sendResponse();
				}
			else
				{
				keepAlive = false;
				}
			}
		else
			{
			keepAlive = false;
			}
		getAccessLog();
		clearHttpData();
		}
	}
catch( SocketException e )
	{
	String message = e.getMessage();
	if( false
	 || message.indexOf( "onnection reset"		) > -1 
	 || message.indexOf( "ocket closed"		) > -1
	 || message.indexOf( "Broken pipe"		) > -1
	 || message.indexOf( "Connection timed out"	) > -1
	 || message.indexOf( "socket write error"	) > -1
	 || message.indexOf( "onnection refused"	) > -1
	 || message.indexOf( " recv failed"		) > -1
	  )
		{
		 //ignore :)
		}
	else
		{
		e.printStackTrace();
		if( timedOut == false )
			{
			getExceptionLog( e );
			}
		}
	}
catch( MHttpException e ) {}	//ignore :)
catch( MHttpIOException e ) {}	//ignroe :)
catch( Exception e )
	{
	e.printStackTrace();
	if( timedOut == false )
		{
		getExceptionLog( e );
		}
	}
finally
	{
	clearHttpData();
	cleanup();
	closeConnection();
	}
}
//--------------------------------------------------------------------------------
private void getExceptionLog( Exception e )
{
exception = e;

notify2( "getExceptionLog", this );
}
//-----------------------------------------------------------------------------------
private void clearHttpData()
{
if( request != null )
	{
	request.deleteTmpFile();
	}
if( response != null )
	{
	response.deleteTmpFile();
	}
request		= null;
response	= null;
}
//-----------------------------------------------------------------------------------
private void resetLoopParameters()
{
time		= 0;
pass		= false;
resultCache	= new MResultCache();
sessionResult	= null;
request		= null;
response	= null;
passRuleSet	= new HashSet();
exception	= null;
logId		= null;
triggeredRule	= null;
pluginClassName	= null;
pluginMessage	= null;
}
//-----------------------------------------------------------------------------------
private void getAccessLog()
{
notify2( "getAccessLog", this );
}
//-----------------------------------------------------------------------------------
private void getAlertLog( MAbstractRule rule )
{
triggeredRule = rule;

notify2( "getAlertLog", this );
}
//--------------------------------------------------------------------------------
private void getPluginAlertLog( MGuardianPlugin plugin, Map pluginResult )
{
pluginClassName = plugin.getClass().getName();
pluginMessage = "";
if( pluginResult.containsKey( "message" ) )
	{
	pluginMessage = ( String )pluginResult.get( "message" );
	}

notify2( "getPluginAlertLog", this );
}
//--------------------------------------------------------------------------------
public String getPluginMessage()
{
return pluginMessage;
}
//--------------------------------------------------------------------------------
public String getPluginClassName()
{
return pluginClassName;
}
//--------------------------------------------------------------------------------
private boolean checkResponse()
throws IOException
{
if( responseRuleList == null
 || pass == true 
  )
	{
	return true;
	}

logId = null;
boolean logged = false;

int count = responseRuleList.size();
for( int i = 0; i < count; ++i )
	{
	MAbstractResponseRule rule = ( MAbstractResponseRule )responseRuleList.get( i );

	if( passRuleSet.contains( rule.getId() ) )
		{
		resultCache.cacheResult( rule, false );
		continue;
		}

	if( rule.matches( response, resultCache ) )
		{
		if( logId == null )
			{
			logId = generateLogId();
			}
		String command	= rule.getCommand();
		String action	= rule.getAction();

			// log
		if( rule.getLogFlag() )
			{
			if( logged == false )
				{
				saveRequest( logId );
				saveResponse( logId );
				logged = true;
				}
			getAlertLog( rule );
			}
			
			// command
		if( rule.hasCommand() )
			{
			execCommand( command, rule, logId );
			}

			// plugin
		List pluginList = rule.getPluginList();
		List pluginArgList = rule.getPluginArgList();
		if( pluginList != null )
			{
			int pluginCount = pluginList.size();
			for( int j = 0; j < pluginCount; ++j )
				{
				MGuardianPlugin plugin = ( MGuardianPlugin )pluginList.get( j );
				String pluginArg = ( String )pluginArgList.get( j );
				Map sessionInfo = new HashMap();
				sessionInfo.put( "request"		, request		);
				sessionInfo.put( "response"		, response		);
				sessionInfo.put( "rule"			, rule			);
				sessionInfo.put( "logId"		, logId			);
				sessionInfo.put( "clientSideSocket"	, clientSideSocket	);
				sessionInfo.put( "serverSideSocket"	, serverSideSocket	);
				sessionInfo.put( "protocol"		, protocol		);
				sessionInfo.put( "arg"			, pluginArg		);
				
					// execute plugin!
				Map pluginResult = plugin.execute( sessionInfo );

				if( pluginResult != null )
					{
					if( pluginResult.containsKey( "log" ) )
						{
						Boolean getLog = ( Boolean )pluginResult.get( "log" );
						if( getLog.booleanValue() == true )
							{
							if( logged == false )
								{	
								saveRequest( logId );
								saveResponse( logId );
								logged = true;
								}
							getPluginAlertLog( plugin, pluginResult );
							}
						}
					if( pluginResult.containsKey( "command" ) )
						{
						String pluginCommand = ( String )pluginResult.get( "command" );
						execCommand( pluginCommand, rule, logId );
						}					
					if( pluginResult.containsKey( "block" ) )
						{
						Boolean doBlock = ( Boolean )pluginResult.get( "block" );
						if( doBlock.booleanValue() == true )
							{
							String pluginMessage = "";
							if( pluginResult.containsKey( "message" ) )
								{
								pluginMessage = ( String )pluginResult.get( "message" );
								}
							sessionResult = "blocked_by_plugin/" + plugin.getClass().getName() + "/" + pluginMessage;
							return false;
							}
						}
					if( pluginResult.containsKey( "pass" ) )
						{
						Boolean doBlock = ( Boolean )pluginResult.get( "pass" );
						if( doBlock.booleanValue() == true )
							{
							pass = true;
							}
						}
					if( pluginResult.containsKey( "request" ) )
						{
						request = ( MHttpRequest )pluginResult.get( "request" );
						}
					if( pluginResult.containsKey( "response" ) )
						{
						response = ( MHttpResponse )pluginResult.get( "response" );
						}
					}
				}
			}			
						
			// action
		if( action.equalsIgnoreCase( MAbstractRule.ACTION_BLOCK ) )
			{
			sessionResult = "blocked_by_rule/" + rule.toString();
			return false;
			}
		else if( action.equalsIgnoreCase( MAbstractRule.ACTION_PASS_ALL ) )
			{
			pass = true;
			}
		else if( action.equalsIgnoreCase( MAbstractRule.ACTION_PASS_RULE ) )
			{
			passRuleSet.addAll( rule.getPassRuleList() );
			}
		
			//pass?
		if( pass == true )
			{
			return true;
			}
		}
	}
return true;
}
//-----------------------------------------------------------------------------------
private boolean checkRequest()
throws IOException
{
if( requestRuleList == null )
	{
	return true;
	}

logId = null;
boolean logged = false;
int count = requestRuleList.size();
for( int i = 0; i < count; ++i )
	{
	MAbstractRequestRule rule = ( MAbstractRequestRule )requestRuleList.get( i );
	
	if( passRuleSet.contains( rule.getId() ) )
		{
		resultCache.cacheResult( rule, false );
		continue;
		}

	if( rule.matches( request , resultCache ) )
		{
		if( logId == null )
			{
			logId = generateLogId();
			}
		String command	= rule.getCommand();
		String action	= rule.getAction();

			// log
		if( rule.getLogFlag() )
			{
			if( logged == false )
				{
				saveRequest( logId );
				logged = true;
				}
			getAlertLog( rule );
			}
			
			// command
		if( rule.hasCommand() )
			{
			execCommand( command, rule, logId );
			}

			// plugin
		List pluginList = rule.getPluginList();
		List pluginArgList = rule.getPluginArgList();
		if( pluginList != null )
			{
			int pluginCount = pluginList.size();
			for( int j = 0; j < pluginCount; ++j )
				{
				MGuardianPlugin plugin = ( MGuardianPlugin )pluginList.get( j );
				String pluginArg = ( String )pluginArgList.get( j );
				Map sessionInfo = new HashMap();
				sessionInfo.put( "request"		, request		);
				sessionInfo.put( "response"		, response		);
				sessionInfo.put( "rule"			, rule			);
				sessionInfo.put( "logId"		, logId			);
				sessionInfo.put( "clientSideSocket"	, clientSideSocket	);
				sessionInfo.put( "serverSideSocket"	, serverSideSocket	);
				sessionInfo.put( "protocol"		, protocol		);
				sessionInfo.put( "arg"			, pluginArg		);
				
					// execute plugin!
				Map pluginResult = plugin.execute( sessionInfo );

				if( pluginResult != null )
					{
					if( pluginResult.containsKey( "log" ) )
						{
						Boolean getLog = ( Boolean )pluginResult.get( "log" );
						if( getLog.booleanValue() == true )
							{
							if( logged == false )
								{	
								saveRequest( logId );
								logged = true;
								}
							getPluginAlertLog( plugin, pluginResult );
							}
						}
					if( pluginResult.containsKey( "command" ) )
						{
						String pluginCommand = ( String )pluginResult.get( "command" );
						execCommand( pluginCommand, rule, logId );
						}
					if( pluginResult.containsKey( "block" ) )
						{
						Boolean doBlock = ( Boolean )pluginResult.get( "block" );
						if( doBlock.booleanValue() == true )
							{
							String pluginMessage = "";
							if( pluginResult.containsKey( "message" ) )
								{
								pluginMessage = ( String )pluginResult.get( "message" );
								}
							sessionResult = "blocked_by_plugin/" + plugin.getClass().getName() + "/" + pluginMessage;
							return false;
							}
						}
					if( pluginResult.containsKey( "pass" ) )
						{
						Boolean doBlock = ( Boolean )pluginResult.get( "pass" );
						if( doBlock.booleanValue() == true )
							{
							pass = true;
							}
						}	
					if( pluginResult.containsKey( "request" ) )
						{
						request = ( MHttpRequest )pluginResult.get( "request" );
						}
					if( pluginResult.containsKey( "response" ) )
						{
						response = ( MHttpResponse )pluginResult.get( "response" );
						}
					if( pluginResult.containsKey( "clientSideSocket" ) )
						{
						clientSideSocket = ( Socket )pluginResult.get( "clientSideSocket" );
						getClientSideStreams();
						}
					if( pluginResult.containsKey( "serverSideSocket" ) )
						{
						closeServerConnection();
						serverSideSocket = ( Socket )pluginResult.get( "serverSideSocket" );
						getServerSideStreams();
						}
					}
				}
			}		
			
			// action
		if( action.equalsIgnoreCase( MAbstractRule.ACTION_BLOCK ) )
			{
			sessionResult = "blocked_by_rule/" + rule.toString();
			return false;
			}
		else if( action.equalsIgnoreCase( MAbstractRule.ACTION_PASS_ALL ) )
			{
			pass = true;
			}
		else if( action.equalsIgnoreCase( MAbstractRule.ACTION_PASS_RULE ) )
			{
			passRuleSet.addAll( rule.getPassRuleList() );
			}
		
		if( pass == true )
			{
			return true;
			}
		}
	}
return true;
}
//--------------------------------------------------------------------------------
private void getServerSideStreams()
throws IOException
{
serverInputStream	= new BufferedInputStream( serverSideSocket.getInputStream() );
serverOutputStream	= serverSideSocket.getOutputStream();
}
//--------------------------------------------------------------------------------
private void getClientSideStreams()
throws IOException
{
clientInputStream	= new BufferedInputStream( clientSideSocket.getInputStream() );
clientOutputStream	= clientSideSocket.getOutputStream();
}
//-----------------------------------------------------------------------------------
private void saveResponse( String logId )
throws IOException
{
MHttpDataLogger.log( response, clientRemoteAddr, logId + "_" + MHttpDataLogger.RES_LOG_SUFFIX );
}
//-----------------------------------------------------------------------------------
private void saveRequest( String logId )
throws IOException
{
MHttpDataLogger.log( request, clientRemoteAddr, logId + "_" + MHttpDataLogger.REQ_LOG_SUFFIX );
}
//-----------------------------------------------------------------------------------
private String generateLogId()
{
StringBuffer strBuf = new StringBuffer( LOGFILENAME_BUFSIZE );
strBuf.append( System.currentTimeMillis() );
strBuf.append( "_" );
strBuf.append( clientRemotePort );
return strBuf.toString();
}
//--------------------------------------------------------------------------------
public String getLogId()
{
return logId;
}
//-----------------------------------------------------------------------------------
private void execCommand( String command, MAbstractRule rule, String logId )
//throws IOException
{
command = command.replaceAll( "%id", rule.getId() );
command = command.replaceAll( "%revision", Integer.toString( rule.getRevision() ) );
command = command.replaceAll( "%name", rule.getName() );
command = command.replaceAll( "%addr", clientRemoteAddr );
command = command.replaceAll( "%port", String.valueOf( clientRemotePort ) );
command = command.replaceAll( "%logId", logId );
command = command.replaceAll( "%logDir", MHttpDataLogger.getSessionLogDirName() );
command = command.replaceAll( "%logDirName", MHttpDataLogger.getSessionLogDirName() );

	// %protocol
if( isHttps() )
	{
	command = command.replaceAll( "%protocol", HTTPS );
	}
else
	{
	command = command.replaceAll( "%protocol", HTTP );
	}
//Runtime.getRuntime().exec( command );
MCommandExecuter.getInstance().addCommand( command );
}
//-----------------------------------------------------------------------------------
public static void setAdditionalResponseHeaderMap( Map in_additionalReponseHeaderMap )
{
additionalResponseHeaderMap = in_additionalReponseHeaderMap;
}
//-----------------------------------------------------------------------------------
public static void setAdditionalRequestHeaderMap( Map in_additionalRequestHeaderMap )
{
additionalRequestHeaderMap = in_additionalRequestHeaderMap;
}
//-----------------------------------------------------------------------------------
public static void setRequestFilter( MHttpFilter in_requestFilter )
{
requestFilter = in_requestFilter;
}
//-----------------------------------------------------------------------------------
public static void setResponseFilter( MHttpFilter in_responseFilter )
{
responseFilter = in_responseFilter;
}
//-----------------------------------------------------------------------------------
private void filterRequest()
{
if( requestFilter != null )
	{
	requestFilter.doFilter( request );
	}
}
//-----------------------------------------------------------------------------------
private void filterResponse()
{
if( responseFilter != null )
	{
	responseFilter.doFilter( response );
	}
}
//-----------------------------------------------------------------------------------
private void connect()
throws IOException
{
if( serverSideSocket == null )
	{
	doConnect();
	getServerSideStreams();
	}
}
//-----------------------------------------------------------------------------------
private void doConnect()
throws IOException
{
SocketAddress sockAddr = new InetSocketAddress( targetHost, targetPort );
serverSideSocket = new Socket();
serverSideSocket.connect( sockAddr, connectTimeOut );
}
//-----------------------------------------------------------------------------------
private void recvRequest()
throws Exception
{
request = new MHttpRequest();
request.init( clientInputStream );
request.setAddr( clientRemoteAddr );
request.setPort( clientRemotePort );

keepAlive = request.isKeepAliveRequest();
}
//-----------------------------------------------------------------------------------
private void modifyRequest()
{
if( replaceHostField == true )
	{
	request.setHeaderValue( "Host", MSession.targetPeer );
	}

	// add additionalHeader
if( additionalRequestHeaderMap != null )
	{
	Iterator i = additionalRequestHeaderMap.keySet().iterator();
	while( i.hasNext() )
		{
		String key = ( String )i.next();
		String value = ( String )additionalRequestHeaderMap.get( key );
		value = replaceSpecialString( value );
		request.setHeaderValue( key, value );
		}
	}
}
//-----------------------------------------------------------------------------------
private String replaceSpecialString( String in )
{
String tmpStr = in;

	// %addr
tmpStr = tmpStr.replaceAll( "%addr", clientRemoteAddr );

	// %protocol
if( isHttps() )
	{
	tmpStr = tmpStr.replaceAll( "%protocol", HTTPS );
	}
else
	{
	tmpStr = tmpStr.replaceAll( "%protocol", HTTP );
	}
return tmpStr;
}
//-----------------------------------------------------------------------------------
private boolean isHttps()
{
return isHttpsFlag;
}
//-----------------------------------------------------------------------------------
private void sendRequest()
throws IOException
{
serverOutputStream.write( request.getHeader() );

if( request.hasBody() )
	{
	MStreamUtil.connectStream( request.getBodyInputStream(), serverOutputStream );
	}
}
//-----------------------------------------------------------------------------------
private void recvResponse()
throws Exception
{
if( request.getMethodType() == MHttpRequest.HEAD )
	{
	response = new MHttpResponse( serverInputStream, true );
	}
else
	{
	response = new MHttpResponse( serverInputStream );
	}

	// 100 continue
if( response.getStatusCode() == 100 )
	{
	response = new MHttpResponse( serverInputStream );
	}

if( response.headerExists( "Connection" )
 && response.getHeaderValue( "Connection" ).equalsIgnoreCase( "close" )
  )
	{
	keepAlive = false;
	}
}
//-----------------------------------------------------------------------------------
private void sendResponse()
throws IOException
{
clientOutputStream.write( response.getHeader() );
if( response.hasBody() )
	{
	MStreamUtil.connectStream( response.getBodyInputStream(), clientOutputStream );
	}
}
//-----------------------------------------------------------------------------------
private void modifyResponse()
{
/*
	// check Location field
if( replaceLocationField )
	{
	if( response.headerExists( "Location" ) )
		{
		String location	= response.getHeaderValue( "Location" );
		if( location.indexOf( targetPeer ) == 7 )
			{
			if( isHttps() )
				{
				location = location.replaceFirst( "^http", "https" );
				response.setHeaderValue( "Location", location );
				}
			}
		}
	}
*/
	// add additionalHeader
if( additionalResponseHeaderMap != null )
	{
	Iterator i = additionalResponseHeaderMap.keySet().iterator();
	while( i.hasNext() )
		{
		String key = ( String )i.next();
		String value = ( String )additionalResponseHeaderMap.get( key );
		response.setHeaderValue( key, value );
		}
	}
}
//-----------------------------------------------------------------------------------
public final void breakCommand()
{
closeConnection();
}
//-----------------------------------------------------------------------------------
public final void closeConnection()
{
closeClientConnection();
closeServerConnection();
}
//-----------------------------------------------------------------------------------
public final void closeClientConnection()
{
try
	{
	if( !clientSideSocket.isClosed() )
		{
		clientSideSocket.close();
		}
	}
catch( IOException e )
	{
	e.printStackTrace();
	}
}
//-----------------------------------------------------------------------------------
public final void closeServerConnection()
{
if( serverSideSocket != null )
	{
	MSocketCloser.getInstance().addSocket( serverSideSocket );
	}
}
//-----------------------------------------------------------------------------------
public static void setResponseRuleList( List in_responseRuleList )
{
MSession.responseRuleList = in_responseRuleList;
}
//-----------------------------------------------------------------------------------
public static void setRequestRuleList( List in_requestRuleList )
{
MSession.requestRuleList = in_requestRuleList;
}
//-----------------------------------------------------------------------------------
public void update()
{
time++;

if( request != null )
	{
	int state = request.getState();

	if( state == MHttpRequest.RECV_REQUEST_HEADER )
		{
		if( time > MSession.requestHeaderTimeOut )
			{
			doTimeOut();
			}
		}
	else if( state == MHttpRequest.RECV_REQUEST_BODY )
		{
		if( time > MSession.requestBodyTimeOut )
			{
			doTimeOut();
			}
		}
	}
}
//-----------------------------------------------------------------------------------
private void doTimeOut()
{
if( timedOut == true )
	{
	return;
	}
else
	{
	timedOut = true;
	getTimeoutLog();
	closeConnection();
	}
}
//-----------------------------------------------------------------------------------
private void getTimeoutLog()
{
notify2( "getTimeoutLog", this );
}
//--------------------------------------------------------------------------------
public String getHostField()
{
String hostField = "-";
if( request != null )
	{
	if( request.headerExists( "Host" ) )
		{
		return request.getHeaderValue( "Host" );
		}
	}
return hostField;
}
//-----------------------------------------------------------------------------------
public static void setTimer( MTimer in_timer )
{
timer = in_timer;
}
//--------------------------------------------------------------------------------
public static void setConnectTimeOut( int i )
{
connectTimeOut = i;
}
//-----------------------------------------------------------------------------------
public static void setRequestBodyTimeOut( int in_requestBodyTimeOut )
{
requestBodyTimeOut = in_requestBodyTimeOut;
}
//-----------------------------------------------------------------------------------
public static void setRequestHeaderTimeOut( int in_requestHeaderTimeOut )
{
requestHeaderTimeOut = in_requestHeaderTimeOut;
}
//-----------------------------------------------------------------------------------
public static void setTargetPeer( String in_targetHost, int in_targetPort )
{
targetHost = in_targetHost;
targetPort = in_targetPort;

if( targetPort == 80 )
	{
	targetPeer = targetHost;
	}
else
	{
	targetPeer = targetHost + ":" + String.valueOf( targetPort );
	}
}
//-----------------------------------------------------------------------------------
public static void setReplaceHostField(boolean b)
{
replaceHostField = b;
}
//-----------------------------------------------------------------------------------
public static void setReplaceLocationField(boolean b)
{
replaceLocationField = b;
}
//-----------------------------------------------------------------------------------
public String getClientLocalAddr()
{
return clientLocalAddr;
}
//-----------------------------------------------------------------------------------
public int getClientLocalPort()
{
return clientLocalPort;
}
//-----------------------------------------------------------------------------------
public String getClientRemoteAddr()
{
return clientRemoteAddr;
}
//-----------------------------------------------------------------------------------
public int getClientRemotePort()
{
return clientRemotePort;
}
//-----------------------------------------------------------------------------------
public String getProtocol()
{
return protocol;
}
//-----------------------------------------------------------------------------------
public MHttpRequest getRequest()
{
return request;
}
//-----------------------------------------------------------------------------------
public MHttpResponse getResponse()
{
return response;
}
//-----------------------------------------------------------------------------------
public String getSessionResult()
{
return sessionResult;
}
//-----------------------------------------------------------------------------------
public void notify2( Object event, Object source )
{
subject2.notify2( event, source );
}
//----------------------------------------------------------------
public void register2( MObserver2 observer )
{
subject2.register2( observer );
}
//----------------------------------------------------------------
public void removeObservers2()
{
subject2.removeObservers2();
}
//----------------------------------------------------------------
public void removeObserver2( MObserver2 observer )
{
subject2.removeObserver2( observer );
}
//-----------------------------------------------------------------------------------
public Exception getException()
{
return exception;
}
//-----------------------------------------------------------------------------------
public Socket getClientSideSocket()
{
return clientSideSocket;
}
//-----------------------------------------------------------------------------------
public MAbstractRule getTriggeredRule()
{
return triggeredRule;
}
//-----------------------------------------------------------------------------------
}