package net.jumperz.app.MGuardian.rule;

import java.io.*;
import java.util.*;
import net.jumperz.util.*;
import net.jumperz.app.MGuardian.plugin.*;

public class MRuleFactory
{
private List requestRuleList	= new ArrayList();
private List responseRuleList	= new ArrayList();
private Map idRuleMap		= new HashMap();

//--------------------------------------------------------------------------------------
public void loadFromFile( String ruleFileName )
throws IOException
{
InputStream in = new FileInputStream( ruleFileName );
try
	{
	loadFromFile( in, ruleFileName );
	}
finally
	{
	in.close();
	}
}
// --------------------------------------------------------------------------------
private void loadFromFile( InputStream in, String ruleFileName )
throws IOException
{
String line = null;
String id = null;
String revision = null;
String name = null;
String type = null;
String patternStr = null;
String conditionStr = null;
String caseSensitiveFlagStr = null;
String logFlagStr = null;
String action = null;
String command = null;
int revisionInt = 0;
boolean caseSensitiveFlag;
boolean logFlag;
boolean negationFlag;
List pluginList = null;
List pluginArgList = null;

String currentFileName	= ruleFileName;
BufferedReader reader	= new BufferedReader( new InputStreamReader( in, MCharset.CS_ISO_8859_1 ) );
Stack readerStateStack	= new Stack();

Set varSet = new HashSet();

for( int lineNumber = 1; ; ++lineNumber )
	{
	line = reader.readLine();
	if( line == null )
		{
		if( readerStateStack.empty() )
			{
			break;
			}
		else
			{
			MReaderState readerState = ( MReaderState )readerStateStack.pop();
			reader		= readerState.getReader();
			lineNumber	= readerState.getCurrentLineNumber();
			currentFileName	= readerState.getCurrentFileName();
			}
		}
	else if( line.indexOf( "include" ) == 0 )
		{
		if( false
		 || id != null
		 || revision != null
		 || name != null 
		 || type != null
		 || patternStr != null
		 || conditionStr != null
		 || caseSensitiveFlagStr != null
		 || logFlagStr != null
		 || action != null
		 || command != null 
		 || pluginList != null
		  )
			{
			throwParseError( currentFileName, lineNumber );
			}
		
		MReaderState readerState = new MReaderState();
		readerState.setCurrentFileName( currentFileName );
		readerState.setCurrentLineNumber( lineNumber );
		readerState.setReader( reader );
		readerStateStack.push( readerState );

		String nextFileName = MRegEx.getMatch( "^include\\s{0,}[\"']{1}(.*)[\"']$", line );
		File nextFile = new File( nextFileName );
		if( !nextFile.isAbsolute() )
			{
			File currentFile	= new File( currentFileName );
			File parentDir		= currentFile.getParentFile();
			nextFile		= new File( parentDir.getCanonicalPath() + "/" + nextFileName );		
			}
		lineNumber = 1;
		currentFileName = nextFile.getAbsolutePath();
		reader = new BufferedReader( new InputStreamReader( new FileInputStream( nextFile ), MCharset.CS_ISO_8859_1 ) );		
		}
	else if( line.matches( "^var\\s{1,}%.*$" ) )
		{
		if( false
		 || id != null
		 || revision != null
		 || name != null 
		 || type != null
		 || patternStr != null
		 || conditionStr != null
		 || caseSensitiveFlagStr != null
		 || logFlagStr != null
		 || action != null
		 || command != null 
		 || pluginList != null
		  )
			{
			throwParseError( currentFileName, lineNumber );
			}

		String varName = MRegEx.getMatch( "^var\\s{1,}(%[-_a-zA-Z0-9]{1,}%)", line ); 
		if( varName.equals( "" ) )
			{
			throwParseError( currentFileName, lineNumber );			
			}

		String varExpression = MRegEx.getMatch( "\"(.*)\"$", line ); 
		if( varExpression.equals( "" ) )
			{
			throwParseError( currentFileName, lineNumber );			
			}

		MVariable var = new MVariable();
		var.setName( varName );
		var.setExpression( varExpression );

		if( varSet.contains( var ) )
			{
				// duplicate variable name
			throwParseError( currentFileName, lineNumber );
			}
		varSet.add( var );
		}	
	else if( line.equalsIgnoreCase( "<rule>" ) )
		{
		if( false
		 || id != null
		 || revision != null
		 || name != null 
		 || type != null
		 || patternStr != null
		 || conditionStr != null
		 || caseSensitiveFlagStr != null
		 || logFlagStr != null
		 || action != null
		 || command != null 
		 || pluginList != null
		  )
			{
			throwParseError( currentFileName, lineNumber );
			}
		}
	else if( line.equalsIgnoreCase( "</rule>" ) )
		{
		if( false
		 || id == null
		 || revision == null
		 || name == null 
		 || type == null
		 || patternStr == null
		 || conditionStr == null
		 || caseSensitiveFlagStr == null
		 || logFlagStr == null
		 || action == null
		 || command == null 
		  )
			{
			throwParseError( currentFileName, lineNumber );
			}

		caseSensitiveFlag = ( caseSensitiveFlagStr.equalsIgnoreCase( "YES" ) );
		logFlag = ( logFlagStr.equalsIgnoreCase( "YES" ) );
		negationFlag = ( conditionStr.equalsIgnoreCase( "NOT MATCH" ) );

		/*
		if( actionStr.equalsIgnoreCase( "NONE" ) )
			{
			action = MAbstractRule.ACTION_NONE;
			}
		else if( actionStr.equalsIgnoreCase( "BLOCK" ) )
			{
			action = MAbstractRule.ACTION_BLOCK;
			}
		else if( actionStr.equalsIgnoreCase( "PASS" ) )
			{
			action = MAbstractRule.ACTION_PASS;
			}
		else
			{
			throwParseError( currentFileName, lineNumber );
			}
		*/

		MAbstractRule rule = null;
		if( type.equalsIgnoreCase( "address" ) )
			{
			rule = new MAddressRule();
			}
		else if( type.equalsIgnoreCase( "requestLine" ) )
			{
			rule = new MRequestLineRule();
			}
		else if( type.equalsIgnoreCase( "requestUri" ) || type.equalsIgnoreCase( "requestUrl" ) )
			{
			rule = new MRequestUriRule();
			}
		else if( type.equalsIgnoreCase( "requestUriPath" ) )
			{
			rule = new MRequestUriPathRule();
			}
		else if( type.equalsIgnoreCase( "requestUriParam" ) )
			{
			rule = new MRequestUriParamRule();
			}
		else if( type.equalsIgnoreCase( "requestUriQuery" ) )
			{
			rule = new MRequestUriQueryRule();
			}
		else if( type.equalsIgnoreCase( "paramValue" ) )
			{
			rule = new MParamValueRule();
			}		
		else if( type.equalsIgnoreCase( "requestHeader" ) )
			{
			rule = new MRequestHeaderRule();
			}
		else if( type.equalsIgnoreCase( "requestBody" ) )
			{
			rule = new MRequestBodyRule();
			}
		else if( type.equalsIgnoreCase( "decodedRequestBody" ) )
			{
			rule = new MDecodedRequestBodyRule();
			}
		else if( type.equalsIgnoreCase( "statusLine" ) )
			{
			rule = new MStatusLineRule();
			}
		else if( type.equalsIgnoreCase( "responseHeader" ) )
			{
			rule = new MResponseHeaderRule();
			}
		else if( type.equalsIgnoreCase( "responseBody" ) )
			{
			rule = new MResponseBodyRule();
			}
		else if( type.equalsIgnoreCase( "responseBody2" ) )
			{
			rule = new MResponseBodyRule2();
			}
		else if( type.equalsIgnoreCase( "responseBodyHead" ) )
			{
			rule = new MResponseBodyHeadRule();
			}
		else if( type.equalsIgnoreCase( "responseBodyLine" ) )
			{
			rule = new MResponseBodyLineRule();
			}
		else if( type.equalsIgnoreCase( "ruleGroup" ) )
			{
			rule = getRuleGroupInstance( patternStr );
			}
		else
			{
			throwParseError( currentFileName, lineNumber );
			}

			// construct rule
		rule.setId( id );
		rule.setRevision( revisionInt );
		rule.setName( name );
		rule.setPattern( patternStr, !caseSensitiveFlag );
		rule.setNegationFlag( negationFlag );
		rule.setLogFlag( logFlag );
		rule.setAction( action );
		rule.setCommand( command );
		rule.setPluginList( pluginList );
		rule.setPluginArgList( pluginArgList );
		rule.init();
		
			// set rule to idRuleMap for ruleGroups
		if( idRuleMap.containsKey( id ) )
			{
			System.err.println( "duplicate rule id" );
			throwParseError( currentFileName, lineNumber );
			}
		idRuleMap.put( id, rule );
		
		int ruleType = rule.getRuleType();
		if( ruleType == MAbstractRule.REQUEST )
			{
			requestRuleList.add( rule );
			}
		else if( ruleType == MAbstractRule.RESPONSE )
			{
			responseRuleList.add( rule );
			}
		else
			{
			throw new IOException( "invalid ruleType" );
			}

		id = null;
		revision = null;
		name = null ;
		type = null;
		patternStr = null;
		conditionStr = null;
		caseSensitiveFlagStr = null;
		logFlagStr = null;
		action = null;
		command = null ;
		pluginList = null;
		pluginArgList = null;
		
		}
	else if( line.indexOf( "name=" ) == 0 )
		{
		if( name != null )
			{
			throwParseError( currentFileName, lineNumber );
			}
		name = getParam( line, "name=" );
		
		/*
			// check " " and "+"
		if( name.indexOf( " " ) > -1 )
			{
			throwParseError( "Blank space is not allowed to use in the name field.", lineNumber );
			}
		if( name.indexOf( "+" ) > -1 )
			{
			throwParseError( "Plus sign is not allowed to use in the name field.", lineNumber );
			}
		*/
		}
	else if( line.indexOf( "id=" ) == 0 )
		{
		if( id != null )
			{
			throwParseError( currentFileName, lineNumber );
			}
		id = getParam( line, "id=" );
		if( !id.matches( "^[-_a-zA-Z0-9]{1,}$" ) )
			{
			throwParseError( currentFileName, lineNumber );
			}
		}
	else if( line.indexOf( "revision=" ) == 0 )
		{
		if( revision != null )
			{
			throwParseError( currentFileName, lineNumber );
			}
		revision = getParam( line, "revision=" );
		if( !revision.matches( "^[0-9]{1,}$" ) )
			{
			throwParseError( currentFileName, lineNumber );
			}
		revisionInt = Integer.parseInt( revision );
		}
	else if( line.indexOf( "type=" ) == 0 )
		{
		if( type != null )
			{
			throwParseError( currentFileName, lineNumber );
			}
		type = getParam( line, "type=" );
		}
	else if( line.indexOf( "pattern=" ) == 0 )
		{
		if( patternStr != null )
			{
			throwParseError( currentFileName, lineNumber );
			}
		patternStr = getParam( line, "pattern=" );
		}
	else if( line.indexOf( "condition=" ) == 0 )
		{
		if( conditionStr != null )
			{
			throwParseError( currentFileName, lineNumber );
			}
		conditionStr = getParam( line, "condition=" );
		}
	else if( line.indexOf( "case_sensitive=" ) == 0 )
		{
		if( caseSensitiveFlagStr != null )
			{
			throwParseError( currentFileName, lineNumber );
			}
		caseSensitiveFlagStr = getParam( line, "case_sensitive=" );
		}
	else if( line.indexOf( "log=" ) == 0 )
		{
		if( logFlagStr != null )
			{
			throwParseError( currentFileName, lineNumber );
			}
		logFlagStr = getParam( line, "log=" );
		}
	else if( line.indexOf( "action=" ) == 0 )
		{
		if( action != null )
			{
			throwParseError( currentFileName, lineNumber );
			}
		action = getParam( line, "action=" );
		}
	else if( line.indexOf( "command=" ) == 0 )
		{
		if( command != null )
			{
			throwParseError( currentFileName, lineNumber );
			}
		command = getParam( line, "command=" );
		}
	else if( line.indexOf( "plugin=" ) == 0 )
		{
		if( pluginList == null )
			{
			pluginList = new ArrayList();
			}
		String pluginStr = getParam( line, "plugin=" );
		String pluginClassName = "";
		String pluginArg = "";
		int index = pluginStr.indexOf( " " );
		if( index == -1 )
			{
			pluginClassName = pluginStr;
			}
		else
			{
			pluginClassName = pluginStr.substring( 0, index );
			pluginArg = pluginStr.substring( index + 1 );
			}
		MGuardianPlugin plugin;
		try
			{
			pluginList.add( MPluginManager.getInstance().getPlugin( pluginClassName ) );
			}
		catch( Exception e )
			{
			e.printStackTrace();
			throwParseError( currentFileName, lineNumber );
			}
		
		if( pluginArgList == null )
			{
			pluginArgList = new ArrayList();
			}
		pluginArgList.add( pluginArg );
		}
	else if( MStringUtil.isComment( line ) )
		{
		}
	else
		{
		throwParseError( currentFileName, lineNumber );
		}
	}
	
resolveVariables( requestRuleList, varSet );
resolveVariables( responseRuleList, varSet );

	//startup plugins
//MPluginManager.getInstance().startup();
}
//--------------------------------------------------------------------------------
private void resolveVariables( List ruleList, Set varSet )
throws IOException
{
	// command
Iterator p = ruleList.iterator();
while( p.hasNext() )
	{
	MAbstractRule rule = ( MAbstractRule )p.next();
	String command = rule.getCommand();
	int count = 0;
	for( ;; ++count )
		{
		boolean found = false;
		Iterator q = varSet.iterator();
		while( q.hasNext() )
			{
			MVariable var = ( MVariable )q.next();
			String name = var.getName();
			String expression = var.getExpression();
			if( command.indexOf( name ) > -1 )
				{
				found = true;
				command = command.replaceAll( name, expression );
				}
			}
		if( !found )
			{
			rule.setCommand( command );
			break;
			}

		if( count > 100 )
			{
				// detect loop :(
			throw new IOException( "variable error : " + command );
			}
		}
	}

	// action
p = ruleList.iterator();
while( p.hasNext() )
	{
	MAbstractRule rule = ( MAbstractRule )p.next();
	String action = rule.getAction();
	int count = 0;
	for( ;; ++count )
		{
		boolean found = false;
		Iterator q = varSet.iterator();
		while( q.hasNext() )
			{
			MVariable var = ( MVariable )q.next();
			String name = var.getName();
			String expression = var.getExpression();
			if( action.indexOf( name ) > -1 )
				{
				found = true;
				action = action.replaceAll( name, expression );
				}
			}
		if( !found )
			{
			if( action.equalsIgnoreCase( MAbstractRule.ACTION_BLOCK )
			 || action.equalsIgnoreCase( MAbstractRule.ACTION_PASS_ALL )
			 || action.equalsIgnoreCase( MAbstractRule.ACTION_NONE )
			  )
				{
				rule.setAction( action );
				}
			else if( action.indexOf( MAbstractRule.ACTION_PASS_RULE ) == 0 )
				{
				String[] array = action.split( "\\s{1,}" );
				if( array.length == 1 )
					{
					throw new IOException( "Invalid action : " + action );
					}
				List passRuleList = new ArrayList( array.length - 1 );
				for( int i = 1; i < array.length; ++i )
					{
					passRuleList.add( array[ i ] );
					}
				rule.setPassRuleList( passRuleList );
				rule.setAction( MAbstractRule.ACTION_PASS_RULE );
				}
			else
				{
				throw new IOException( "Invalid action : " + action );
				}
			break;
			}

		if( count > 100 )
			{
				// detect loop :(
			throw new IOException( "variable error : " + action );
			}
		}
	}

}
//--------------------------------------------------------------------------------------
private void throwParseError( String ruleFileName, int lineNumber )
throws IOException
{
throw new IOException( "Parse error. " + ruleFileName + " line: " + lineNumber );
}
//--------------------------------------------------------------------------------------
private String getParam( String line, String paramName )
{
return line.substring( paramName.length() );
}
//--------------------------------------------------------------------------------------
public List getRequestRuleList()
{
return requestRuleList;
}
//--------------------------------------------------------------------------------------
public List getResponseRuleList()
{
return responseRuleList;
}
//--------------------------------------------------------------------------------------
private MAbstractRule getRuleGroupInstance( String patternStr )
{
MAbstractRule ruleGroup;
boolean containsResponseRule = false;

String[] ruleIdList = patternStr.split( " {0,}\\+ {0,}" );
int ruleCount = ruleIdList.length;

ArrayList ruleList = new ArrayList( ruleCount );

for( int i = 0; i < ruleCount; ++i )
	{
	MAbstractRule rule = ( MAbstractRule )( idRuleMap.get( ruleIdList[ i ] ) );
	ruleList.add( rule );
	if( rule.getRuleType() == MAbstractRule.RESPONSE )
		{
		containsResponseRule = true;
		}
	}

if( containsResponseRule )
	{
	ruleGroup = new MRuleGroup( ruleList );
	}
else
	{
	ruleGroup = new MRequestRuleGroup( ruleList );
	}

return ruleGroup;
}
//--------------------------------------------------------------------------------------
}