package net.jumperz.net;

import java.io.*;
import java.util.*;
import java.net.*;
import net.jumperz.util.*;
import net.jumperz.net.exception.*;
import net.jumperz.io.*;

public abstract class MHttpData
{
protected List headerList;
//protected InputStream netInputStream;
protected BufferedInputStream bufferedInputStream;
protected int headerLength = 0;
protected int pos;
protected MBuffer bodyBuffer;
protected boolean hasBodyFlag = false;

//protected byte[] body;

protected static final int HEADER_BUFSIZE	= 1024;
protected static final int BODY_BUFSIZE		= 4096;
private static final int MAX_HEADER_COUNT	= 200;
protected static final byte[] CRLF		= { (byte)0x0D, (byte)0x0A };
protected static boolean strictDelimiter = true;

public abstract byte[] getHeader();
protected abstract void recvBodyUntilDisconnected() throws IOException;
//-------------------------------------------------------------------------------------------
public MHttpData()
{
headerList = new ArrayList();
}
//-------------------------------------------------------------------------------------------
public final void addHeaderValue( String name, String value )
{
headerList.add( name + ": " + value );
}
//-------------------------------------------------------------------------------------------
public synchronized final void setHeaderValue( String name, String value )
{
int index = -1;

	// get index
int count = headerList.size();
for( int i = 0; i < count; ++i )
	{
	String header = ( String )headerList.get( i );
	if( header.toUpperCase().indexOf( name.toUpperCase() + ":" ) == 0 )
		{
		index = i;
		break;
		}
	}

if( index > -1 )
	{
	removeHeaderValue( name );
	headerList.add( index, name + ": " + value );
	}
else
	{
	headerList.add( name + ": " + value );
	}


/*
removeHeaderValue( name );
headerList.add( name + ": " + value );
*/
}
// --------------------------------------------------------------------------------
public static void setStrictDelimiter( boolean b )
{
strictDelimiter = b;
}
// --------------------------------------------------------------------------------
protected MLineReader getLineReader()
{
if( strictDelimiter )
	{
	return new MCRLFLineReader();
	}
else
	{
	return new MAdvancedLineReader();
	}
}
//-------------------------------------------------------------------------------------------
public final String getHeaderValue( String name )
{
int count = headerList.size();
for( int i = 0; i < count; ++i )
	{
	String header = ( String )headerList.get( i );
	if( header.toUpperCase().indexOf( name.toUpperCase() + ":" ) == 0 )
		{
		String value = MRegEx.getMatch( "[ ]*(.*)$", header.substring( name.length() + 1 ) );
		return value;
		}
	}
return null;
}
// --------------------------------------------------------------------------------
public final List getHeaderValueList( String name )
{
List l = new ArrayList();
int count = headerList.size();
for( int i = 0; i < count; ++i )
	{
	String header = ( String )headerList.get( i );
	if( header.toUpperCase().indexOf( name.toUpperCase() + ":" ) == 0 )
		{
		String value = MRegEx.getMatch( "[ ]*(.*)$", header.substring( name.length() + 1 ) );
		l.add( value );
		}
	}

return l;
}
//-------------------------------------------------------------------------------------------
public synchronized final void removeHeaderValue( String name )
{
int count = headerList.size();
for( int i = 0; i < count; ++i )
	{
	String header = ( String )headerList.get( i );
	if( header.toUpperCase().indexOf( name.toUpperCase() + ":" ) == 0 )
		{
		headerList.remove( i );
		--count;
		--i;
		}
	}
}
//-------------------------------------------------------------------------------------------
public final boolean headerExists( String name )
{
int count = headerList.size();
for( int i = 0; i < count; ++i )
	{
	String header = ( String )headerList.get( i );
	if( header.toUpperCase().indexOf( name.toUpperCase() + ":" ) == 0 )
		{
		return true;
		}
	}
return false;
}
//-------------------------------------------------------------------------------------------
protected final void readHeaderFields( MLineReader reader )
throws IOException
{
StringBuffer headerStrBuf = new StringBuffer( HEADER_BUFSIZE );
String line;

	// read all header fields
for( int i = 0;; ++i )
	{
	line = reader.readLine();
	if( line == null )
		{
		throw new IOException( "Invalid HTTP data" );
		}

	headerLength += line.length() + reader.getLastDelimiterSize();
	
	if( line.equals( "" ) )
		{
		bufferedInputStream.reset();
		bufferedInputStream.skip( headerLength );
		bufferedInputStream.mark( Integer.MAX_VALUE );
		break;
		}
	else
		{
		headerStrBuf.append( line );
		headerStrBuf.append( "\r\n" );
		}
	
		// for DoS attack 
	if( i > MHttpData.MAX_HEADER_COUNT )
		{
		throw new IOException( "Too many header fields." );
		}
	}

String headerStr = headerStrBuf.toString();
if( headerStr.equals( "" ) )
	{
		// no header fields
	return;
	}

String[] headerArray = headerStr.split( "\r\n" );

for( int i = 0; i < headerArray.length; ++i )
	{
	if( headerArray[ i ].indexOf( ":" ) == -1 )
		{
		//System.err.println( headerArray[ i ] );
		throw new IOException( "Invalid HTTP header ( multiple line header field is not allowed )" );
		}
	headerList.add( headerArray[ i ] );
	}

/* old code =)
headerStr = headerStr.replaceAll( "\r\n ", "\r " );
headerStr = headerStr.replaceAll( "\r\n\t", "\r\t" );
String[] headerArray = headerStr.split( "\r\n" );

for( int i = 0; i < headerArray.length; ++i )
	{
	String workStr	= headerArray[ i ].replaceAll( "\r\t" , "\r\n\t" );
	workStr		= workStr.replaceAll( "\r ", "\r\n " );
	if( workStr.indexOf( ":" ) == -1 )
		{
		System.err.println( workStr );
		throw new IOException( "Invalid HTTP data" );
		}
	headerList.add( workStr );
	}
*/
}
//-------------------------------------------------------------------------------------------
protected final void recvBody()
throws IOException
{
try
	{
	if( headerExists( "Transfer-Encoding" ) )
		{
		String transferEncoding = getHeaderValue( "Transfer-Encoding" );
		if( transferEncoding.equals( "chunked" ) )
			{
			recvChunkedBody();
			}
		else
			{
			throw new IOException( "Unsupported Transfer-Encoding :" + transferEncoding );
			}
		}
	else if( headerExists( "Content-length" ) )
		{
		recvBodyByContentLength();
		}
	else
		{
			// recv data until disconnected by server
		recvBodyUntilDisconnected();
		}
	}
catch( OutOfMemoryError e )
	{
	e.printStackTrace();
	throw new MHttpException( 413, "Request entity too large" );
	}
}
// --------------------------------------------------------------------------------
protected int getContentLength()
throws MHttpIOException
{
int contentLength = -1;
if( headerExists( "Content-Length" ) )
	{
	String value = getHeaderValue( "Content-Length" );
	try
		{
		contentLength = Integer.parseInt( value );
		if( contentLength < 0 )
			{		
			throw new MHttpIOException( "Invalid Content-Length" );
			}
		}
	catch( NumberFormatException e )
		{
		throw new MHttpIOException( "Invalid Content-Length" );
		}
	}
return contentLength;
}
//-------------------------------------------------------------------------------------------
protected final void recvBodyByContentLength()
throws IOException
{
bodyBuffer = new MBuffer();
int received;
int remain = getContentLength();

int canRead	= BODY_BUFSIZE;
byte[] buffer	= new byte[ BODY_BUFSIZE ];

if( remain == 0 )
	{
	hasBodyFlag = false;
	return;
	}

while( remain > 0 )
	{
	if( remain < BODY_BUFSIZE )
		{
		canRead = remain;
		}

	bufferedInputStream.mark( canRead );

	received = bufferedInputStream.read( buffer, 0, canRead );
	if( received <= 0 )
		{
			// HEAD, etc
		break;
		}	
	remain -= received;

	bufferedInputStream.reset();
	bufferedInputStream.skip( received );

	bodyBuffer.write( buffer, received );
	}
bufferedInputStream.mark( Integer.MAX_VALUE );
}
//-------------------------------------------------------------------------------------------
protected final void recvChunkedBody()
throws IOException
{
	// http://www.w3.org/Protocols/rfc2616/rfc2616-sec19.html#sec19.4.6

bodyBuffer = new MBuffer();
	
	// mark
bufferedInputStream.mark( Integer.MAX_VALUE );

	// read chunk-size, chunk-extension (if any) and CRLF
int chunkSize		= readChunkSize( bufferedInputStream, bodyBuffer );

	// while (chunk-size > 0)
while( chunkSize > 0 )
	{
		// read chunk-data
	int received;
	int remain	= chunkSize;
	int canRead	= BODY_BUFSIZE;
	byte[] buffer	= new byte[ BODY_BUFSIZE ];

	while( remain > 0 )
		{	
		if( remain < BODY_BUFSIZE )
			{
			canRead = remain;
			}

		received = bufferedInputStream.read( buffer, 0, canRead );
		if( received <= 0 )
			{
			throw new IOException( "Failed to read the chunked data" );
			}
		remain	-= received;
		
			// append chunk-data to entity-body
		bodyBuffer.write( buffer, received );
		}

		// and CRLF
	readCRLF( bufferedInputStream, bodyBuffer );

		// reset, skip and mark
	bufferedInputStream.reset();
	bufferedInputStream.skip( chunkSize + 2 );
	bufferedInputStream.mark( Integer.MAX_VALUE );
	
		// read chunk-size and CRLF
	chunkSize		= readChunkSize( bufferedInputStream, bodyBuffer );
	}
	
	// read entity-header
String trailer = readTrailer( bufferedInputStream, bodyBuffer );

	// while (entity-header not empty)
while( !trailer.equals( "" ) )
	{	
		// read entity-header
	trailer = readTrailer( bufferedInputStream, bodyBuffer );
	}
}
//-------------------------------------------------------------------------------------------
/*
 * Caution: inputStream must reset, skipped and marked
 */
private final String readTrailer( BufferedInputStream inputStream, MBuffer bodyBuffer )
throws IOException
{
MLineReader reader = getLineReader();
reader.setInputStream( inputStream );

String trailer = reader.readLine();
if( trailer == null )
	{
	return "";
	}

byte[] buffer = trailer.getBytes( MCharset.CS_ISO_8859_1 ); 
bodyBuffer.write( buffer );
bodyBuffer.write( CRLF );

	// reset and skip
inputStream.reset();
inputStream.skip( trailer.length() + 2 );
inputStream.mark( Integer.MAX_VALUE );

return trailer;
}
//-------------------------------------------------------------------------------------------
/*
 * Caution: inputStream must reset, skipped and marked
 */
private final int readChunkSize( BufferedInputStream inputStream, MBuffer bodyBuffer )
throws IOException
{	
MLineReader reader = getLineReader();
reader.setInputStream( inputStream );

String line		= reader.readLine();
if( line == null )
	{
	return 0;
	}

byte[] buffer = line.getBytes( MCharset.CS_ISO_8859_1 );
bodyBuffer.write( buffer );
bodyBuffer.write( CRLF );
String chunkSizeStr	= MRegEx.getMatch( "^[0-9A-Fa-f]{1,}", line );

	// reset, skip and mark
inputStream.reset();
inputStream.skip( line.length() + 2 );
inputStream.mark( Integer.MAX_VALUE );

return Integer.parseInt( chunkSizeStr, 16 );
}
//-------------------------------------------------------------------------------------------
private final void readCRLF( InputStream inputStream, MBuffer bodyBuffer )
throws IOException
{
int CR = inputStream.read();
if( CR != 0x0D )
	{
	throw new IOException( "Invalid CR" );
	}
int LF = inputStream.read();
if( LF != 0x0A )
	{
	throw new IOException( "Invalid LF" );
	}
bodyBuffer.write( CRLF );
}
//-------------------------------------------------------------------------------------------
public final InputStream getBodyInputStream()
{
return bodyBuffer.getInputStream();
}
// --------------------------------------------------------------------------------
public final byte[] getBodyAsByte()
throws IOException
{
return MStreamUtil.streamToBytes( getBodyInputStream() );
}
// --------------------------------------------------------------------------------
public final String getBodyAsString()
throws IOException
{
return MStreamUtil.streamToString( getBodyInputStream() ); 
}
//-------------------------------------------------------------------------------------------
public final synchronized ArrayList getHeaderFieldList()
{
int count = headerList.size();
ArrayList headerFieldList = new ArrayList( count );
for( int i = 0; i < count; ++i )
	{
	String header = ( String )headerList.get( i );
	headerFieldList.add( MRegEx.getMatch( "([^:]+):", header ) );
	}
return headerFieldList;
}
//-------------------------------------------------------------------------------------------
public final List getHeaderList()
{
return new ArrayList( headerList );
}
//--------------------------------------------------------------------------------
public final void setHeaderList( List list )
{
headerList = list;
}
//-------------------------------------------------------------------------------
public final boolean hasBody()
{
return hasBodyFlag;
}
//-------------------------------------------------------------------------------------------
public final void deleteTmpFile()
{
if( bodyBuffer != null )
	{
	bodyBuffer.deleteTmpFile();
	}
}
// --------------------------------------------------------------------------------
private void resetBuffer()
throws IOException
{
if( bodyBuffer != null )
	{
	bodyBuffer.close();
	}
}
//--------------------------------------------------------------------------------
public final void setBody( byte[] data )
throws IOException
{
resetBuffer();

bodyBuffer = new MBuffer();
bodyBuffer.write( data );
hasBodyFlag = true;
}
// --------------------------------------------------------------------------------
public final void setBody( String str, String charset )
throws IOException
{
resetBuffer();

bodyBuffer = new MBuffer();
bodyBuffer.write( str.getBytes( charset ) );
hasBodyFlag = true;
}
//--------------------------------------------------------------------------------
public final void setBody( String str )
throws IOException
{
setBody( str, MCharset.CS_ISO_8859_1 );
}
// --------------------------------------------------------------------------------
public byte[] toByteArray()
{
ByteArrayOutputStream baos = toByteArrayOutputStream();
return baos.toByteArray();
}
// --------------------------------------------------------------------------------
private ByteArrayOutputStream toByteArrayOutputStream()
{
ByteArrayOutputStream baos = new ByteArrayOutputStream( 512 );
try
	{
	baos.write( getHeader() );
	if( hasBody() )
		{
		MStreamUtil.connectStream( getBodyInputStream(), baos );
		}
	}
catch( IOException e )
	{
	e.printStackTrace();
	}
return baos;
}
//--------------------------------------------------------------------------------
public String toString( String enc )
{
ByteArrayOutputStream baos = toByteArrayOutputStream();
String s = null;
try
	{
	s = baos.toString( enc );
	}
catch( UnsupportedEncodingException e )
	{
	e.printStackTrace();
	}
return s;
}
// --------------------------------------------------------------------------------
public String toString()
{
return toString( MCharset.CS_ISO_8859_1 );
}
//--------------------------------------------------------------------------------
}