package net.jumperz.net;

import java.io.*;
import java.net.*;
import java.util.*;
import net.jumperz.io.*;
import net.jumperz.util.*;
import net.jumperz.net.exception.*;

public final class MHttpRequest
extends MHttpData
{
private int state;
private int methodType;
private String method;
private String uri;
private String version;
//private String requestLine;
private String addr;
private int port;

public static final int GET	=  0;
public static final int POST	=  1;
public static final int HEAD	=  2;
public static final int PUT	=  3;
public static final int DELETE	=  4;
public static final int TRACE	=  5;
public static final int CONNECT	=  6;
public static final int OPTIONS	=  7;

public static final int OTHER	= -1;

public static final int RECV_REQUEST_HEADER	= 0;
public static final int RECV_REQUEST_BODY	= 1;
public static final int REQUEST_COMPLETE	= 2;

public static final int DEFAULT_METHOD_TYPE	= GET;
public static final String DEFAULT_METHOD	= "GET";
public static final String DEFAULT_URI		= "/";
public static final String DEFAULT_VERSION	= "HTTP/1.0";
//public static final String DEFAULT_REQUEST_LINE	= "GET / HTTP/1.0";

//--------------------------------------------------------------------------------
public MHttpRequest()
{
methodType	= DEFAULT_METHOD_TYPE;
method		= DEFAULT_METHOD;
uri		= DEFAULT_URI;
version		= DEFAULT_VERSION;
}
//--------------------------------------------------------------------------------
public MHttpRequest( String s )
throws IOException
{
BufferedInputStream i = new BufferedInputStream( new ByteArrayInputStream( s.getBytes( MCharset.CS_ISO_8859_1 ) ) );
init( i );
}
// --------------------------------------------------------------------------------
public MHttpRequest( byte[] buffer )
throws IOException
{
BufferedInputStream i = new BufferedInputStream( new ByteArrayInputStream( buffer ) );
init( i );
}
//--------------------------------------------------------------------------------
public MHttpRequest( BufferedInputStream in_bufferedInputStream )
throws IOException
{
init( in_bufferedInputStream );
}
//-------------------------------------------------------------------------------
public void init( BufferedInputStream in_bufferedInputStream )
throws IOException
{
state = MHttpRequest.RECV_REQUEST_HEADER;
bufferedInputStream = in_bufferedInputStream;
recvHeader();
setHasBodyFlag();

if( hasBodyFlag )
	{
	state = MHttpRequest.RECV_REQUEST_BODY;
	recvBody();
	}
state = MHttpRequest.REQUEST_COMPLETE;
}
//-------------------------------------------------------------------------------
private void recvHeader()
throws IOException
{
bufferedInputStream.mark( Integer.MAX_VALUE );
bufferedInputStream.reset();

MLineReader reader = getLineReader();
reader.setInputStream( bufferedInputStream );

	//1st line of HTTP request header
String line = reader.readLine();

if( line == null || line.equals( "" ) )
	{
	throw new MHttpIOException( "Couldn't receive HTTP request" );
	}	
splitRequestLine( line );
headerLength += line.length() + reader.getLastDelimiterSize();

readHeaderFields( reader );
}
//--------------------------------------------------------------------------------
private void splitRequestLine( String line )
throws IOException
{
	//block "GET / HTTP/1.0 "
if( line.charAt( line.length() -1 ) == ' ' )
	{
	throw new MHttpIOException( "Invalid request line:" + line );
	}

String[] requestLineArray = line.split( " " );
if( requestLineArray.length != 3 )
	{
	throw new MHttpIOException( "Invalid request line:" + line );
	}

setMethod( requestLineArray[ 0 ] );
uri	= requestLineArray[ 1 ];
version	= requestLineArray[ 2 ];
}
//-------------------------------------------------------------------------------
public int getMethodType()
{
return methodType;
}
//-------------------------------------------------------------------------------
public byte[] getHeader()
{
ByteArrayOutputStream bufferStream = null;
try
	{
		// request line
	bufferStream = new ByteArrayOutputStream( HEADER_BUFSIZE );
	bufferStream.write( method.getBytes( MCharset.CS_ISO_8859_1 ) );
	bufferStream.write( (byte)0x20 );
	bufferStream.write( uri.getBytes( MCharset.CS_ISO_8859_1 ) );
	bufferStream.write( (byte)0x20 );
	bufferStream.write( version.getBytes( MCharset.CS_ISO_8859_1 ) );
	bufferStream.write( CRLF );
	
		// fields
	int count = headerList.size();
	for( int i = 0; i < count; ++i )
		{
		bufferStream.write( ( ( String )headerList.get( i ) ).getBytes( MCharset.CS_ISO_8859_1 ) );
		bufferStream.write( CRLF );
		}
	
		// blank line
	bufferStream.write( CRLF );
	}
catch( IOException e )
	{
	//MLogger.getInstance().Log( e.toString() );
	e.printStackTrace();
	}
return bufferStream.toByteArray();
}
//-------------------------------------------------------------------------------
public String getUri()
{
return uri;
}
//-------------------------------------------------------------------------------
public void setVersion( String in_version )
{
version = in_version;
}
//-------------------------------------------------------------------------------
public String getVersion()
{
return version;
}
//-------------------------------------------------------------------------------
public int getHeaderLength()
{
return headerLength;
}
//-------------------------------------------------------------------------------
public void setUri(String uri)
{
this.uri = uri;
}
//-------------------------------------------------------------------------------
public void setMethod( String in_method )
{
method = in_method;

if( method.equals( "GET" ) )
	{
	methodType = GET;
	}
else if( method.equals( "POST" ) )
	{
	methodType = POST;
	}
else if( method.equals( "HEAD" ) )
	{
	methodType = HEAD;
	}
else if( method.equals( "PUT" ) )
	{
	methodType = PUT;
	}
else if( method.equals( "DELETE" ) )
	{
	methodType = DELETE;
	}
else if( method.equals( "TRACE" ) )
	{
	methodType = TRACE;
	}
else if( method.equals( "CONNECT" ) )
	{
	methodType = CONNECT;
	}
else if( method.equals( "OPTIONS" ) )
	{
	methodType = OPTIONS;
	}

else
	{
	methodType = OTHER;
	}
}
// --------------------------------------------------------------------------------
private void setHasBodyFlag()
throws MHttpIOException
{
if( headerExists( "Transfer-Encoding" ) )
	{
	hasBodyFlag = true;
	return;
	}
int contentLength = getContentLength();
hasBodyFlag = ( contentLength > 0 );
}
//-------------------------------------------------------------------------------
public String getRequestLine()
{
StringBuffer strBuf = new StringBuffer( 100 );
strBuf.append( method );
strBuf.append( " " );
strBuf.append( uri );
strBuf.append( " " );
strBuf.append( version );
return strBuf.toString();
}
//--------------------------------------------------------------------------------
public boolean isKeepAliveRequest()
{
boolean keepAlive;
if( version.equals( "HTTP/1.1" ) )
	{
	keepAlive = true;
	
	if( headerExists( "Connection" )
	 && getHeaderValue( "Connection" ).equalsIgnoreCase( "close" )
	  )
		{
		keepAlive = false;
		}
	}
else
	{
	keepAlive = false;

	if( headerExists( "Connection" )
	 && getHeaderValue( "Connection" ).equalsIgnoreCase( "Keep-Alive" )
	  )
		{
		keepAlive = true;
		}	
	}
return keepAlive;
}
//-------------------------------------------------------------------------------
public int getState()
{
return state;
}
//-------------------------------------------------------------------------------
public String getAddr()
{
return addr;
}
//-------------------------------------------------------------------------------
public int getPort()
{
return port;
}
//-------------------------------------------------------------------------------
public void setPort( int in_port )
{
port = in_port;
}
//-------------------------------------------------------------------------------
public void setAddr( String in_addr )
{
addr = in_addr;
}
//--------------------------------------------------------------------------------
public String getMethod()
{
return method;
}
//-------------------------------------------------------------------------------
protected void recvBodyUntilDisconnected()
throws IOException
{
throw new MHttpException( 411, "Length Required" );
}
// --------------------------------------------------------------------------------
}
